// Example code for homework 3.

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rigidbody.h"

// global parameters for simulation
int num_rb;
int end_seconds; // final time
const float frames_per_second=24;
int collision_steps_per_frame;
float epsilon; // coefficient of restitution for elastic cases
float friction;
// you may want to add more parameters for initial conditions of your objects

// global variables
int frame=0;
float sim_t=0; // current time

// functions
void read_parameters(const char *filename);
void init_rigid_bodies(RigidBody **rb);
void write_objects(const char *filename, const RigidBody *rb);
void advance_with_collisions(RigidBody *rb, float end_time);

// start of definitions
int main(int argc, char **argv)
{
   int i, j;
   const char *output_directory=".";
   char filename[1000];
   RigidBody *rb; // at least 4 blocks and a sphere
   float end_t;

   if(argc<2 || argc>3){
      fprintf(stderr,"Need arguments: parameter-file [optional-output-directory]\n");
      return 1;
   }else if(argc==3)
      output_directory=argv[2];

   read_parameters(argv[1]);
   init_rigid_bodies(&rb);
   for(i=0; i<num_rb; ++i){
      sprintf(filename, "%s/object%02d.levelset", output_directory, i);
      write_level_set(filename, rb[i].phi);
      sprintf(filename, "%s/object%02d.mesh", output_directory, i);
      write_tri_mesh(filename, rb[i].mesh);
   }

   sprintf(filename, "%s/rigidbodies%04d", output_directory, frame);
   write_objects(filename, rb);

   while(frame/frames_per_second < end_seconds){
      ++frame;
      for(j=1; j<=collision_steps_per_frame; ++j){
         end_t=(frame+j/(float)collision_steps_per_frame)/frames_per_second;
         advance_with_collisions(rb, end_t);
         sim_t=end_t;
      }
      sprintf(filename, "%s/rigidbodies%04d", output_directory, frame);
      write_objects(filename, rb);
      fputc(frame%10 ? '+' : '#', stdout); fflush(stdout);
   }
   return 0;
}

void read_parameters(const char *filename)
{
   FILE *fp=fopen(filename,"rt");
   fscanf(fp, "%d", &end_seconds);
   fscanf(fp, "%d", &collision_steps_per_frame);
   fscanf(fp, "%f %f", &epsilon, &friction);
   // @@@ you may want to read in other parameters to control how your rigid bodies are initially set up
   //  (and save on recompiling)
   fclose(fp);
}

void init_rigid_bodies(RigidBody **rb)
{
   // @@@ fill in code here which generates your stack of blocks and the sphere,
   // by calling the appropriate make_* functions from rigidbody.h, levelset.h, and trimesh.h
}

void write_objects(const char *filename, const RigidBody *rb)
{
   int i;
   FILE *fp;
   fp=fopen(filename, "wt");
   if(!fp) return;
   fprintf(fp, "%d\n", num_rb);
   for(i=0; i<num_rb; ++i)
      write_rigid_body_parameters(fp, rb+i);
}

void advance_with_collisions(RigidBody *rb, float end_t)
{
   // @@@ fill in code here that advances all the rigid bodies to end_t (from sim_t),
   // with gravity (acceleration=9.8m/s^2 downwards in y), checking for and resolving
   // collisions with both each other and the ground (y=0).
   // You may want to add extra functions similar to those in homework 2.
}

