#ifndef RIGIDBODY_H
#define RIGIDBODY_H

#include <stdio.h>
#include "levelset.h"
#include "trimesh.h"

typedef struct{
   float x[3]; // position (of centre-of-mass)
   float q[4]; // rotation (a unit-length quaternion)
   float v[3]; // velocity (of centre-of-mass)
   float L[3]; // angular momentum
   float M; // total mass
   float inertiaT[3]; // diagonalized inertia tensor
   float inertiaTinv[3]; // inverse
   LevelSet *phi; // level set representation in object space
   TriMesh *mesh; // triangle-mesh representation in object space
} RigidBody;

void read_rigid_body_parameters(FILE *fp, RigidBody *rb);
void write_rigid_body_parameters(FILE *fp, const RigidBody *rb);
void make_rigid_body_parameters_of_block(RigidBody *rb, float xrange, float yrange, float zrange, float mass);
void make_rigid_body_parameters_of_sphere(RigidBody *rb, float radius, float mass);

// Note: for getting the geometry in and out of a rigid body, use the read_ and write_ functions in
// levelset.* and trimesh.*
// Also note: if you happen to use general geometry from other sources, note that computing the diagonalized inertia
// tensor may involve translating and rotating the mesh and level set.

#endif
