#include <stdio.h>
#include <stdlib.h>
#include "trimesh.h"
#include "vectors.h"

void read_tri_mesh(const char *filename, TriMesh *mesh)
{
   FILE *fp;
   int i;

   fp=fopen(filename, "rt");
   if(!fp) return;
   fscanf(fp, "%d", &mesh->nv);
   mesh->x=(float*)malloc(3*mesh->nv*sizeof(float));
   for(i=0; i<3*mesh->nv; ++i)
      fscanf(fp, "%g", mesh->x+i);
   fscanf(fp, "%d", &mesh->nt);
   mesh->tri=(int*)malloc(3*mesh->nt*sizeof(int));
   for(i=0; i<3*mesh->nt; ++i)
      fscanf(fp, "%d", mesh->tri+i);
   fclose(fp);
   mesh->normals=0; // by default we don't compute these
}

void compute_vertex_normals(TriMesh *mesh)
{
   int i;

   mesh->normals=(float*)calloc(3*mesh->nv,sizeof(float));
   for(i=0; i<mesh->nt; ++i){
      int a=mesh->tri[3*i], b=mesh->tri[3*i+1], c=mesh->tri[3*i+2];
      float normal[3];
      normal[0]=(mesh->x[3*b+1]-mesh->x[3*a+1])*(mesh->x[3*c+2]-mesh->x[3*a+2])-(mesh->x[3*b+2]-mesh->x[3*a+2])*(mesh->x[3*c+1]-mesh->x[3*a+1]);
      normal[1]=(mesh->x[3*b+2]-mesh->x[3*a+2])*(mesh->x[3*c+0]-mesh->x[3*a+0])-(mesh->x[3*b+0]-mesh->x[3*a+0])*(mesh->x[3*c+2]-mesh->x[3*a+2]);
      normal[2]=(mesh->x[3*b+0]-mesh->x[3*a+0])*(mesh->x[3*c+1]-mesh->x[3*a+1])-(mesh->x[3*b+1]-mesh->x[3*a+1])*(mesh->x[3*c+0]-mesh->x[3*a+0]);
      normalize_v(normal);
      add_v(mesh->normals+3*a, normal);
      add_v(mesh->normals+3*b, normal);
      add_v(mesh->normals+3*c, normal);
   }
   for(i=0; i<mesh->nv; ++i)
      normalize_v(mesh->normals+3*i);
}

void write_tri_mesh(const char *filename, TriMesh *mesh)
{
   FILE *fp;
   int i;

   fp=fopen(filename, "wt");
   if(!fp) return;
   fprintf(fp, "%d\n", mesh->nv);
   for(i=0; i<mesh->nv; ++i)
      fprintf(fp, "%.7g %.7g %.7g\n", mesh->x[3*i], mesh->x[3*i+1], mesh->x[3*i+2]);
   fprintf(fp, "%d\n", mesh->nt);
   for(i=0; i<mesh->nt; ++i)
      fprintf(fp, "%d %d %d\n", mesh->tri[3*i], mesh->tri[3*i+1], mesh->tri[3*i+2]);
   fclose(fp);
}

void make_tri_mesh_of_block(TriMesh *mesh, float xrange, float yrange, float zrange, float spacing)
{
   // this code is up to you: create a triangle mesh of the block from
   // -xrange/2 to xrange/2, -yrange/2 to yrange/2, -zrange/2 to zrange/2
   // where the distance between a vertex and its closest neighbour is roughly spacing.
}

void make_tri_mesh_of_sphere(TriMesh *mesh, float radius, float spacing)
{
   int i;
   make_tri_mesh_of_block(mesh, 2*radius, 2*radius, 2*radius, spacing);
   for(i=0; i<mesh->nv; ++i){
      float scale=radius/sqrt(mesh->x[3*i]*mesh->x[3*i]+mesh->x[3*i+1]*mesh->x[3*i+1]+mesh->x[3*i+2]*mesh->x[3*i+2]);
      mesh->x[3*i]*=scale;
      mesh->x[3*i+1]*=scale;
      mesh->x[3*i+2]*=scale;
   }
}

